@@ -21,7 +21,7 @@
     $Id$
 */
 
-/// \file config_setup_client.cc
+/// @file config/setup/config_setup_client.cc
 #define GRB_LOG_FAC GrbLogFacility::config
 
 #include "config_setup_client.h" // API
@@ -43,9 +43,10 @@
 #include <iterator>
 #include <numeric>
 
-/// \brief Creates an array of ClientConfig objects from a XML nodeset.
-/// \param element starting element of the nodeset.
+/// @brief Creates an array of ClientConfig objects from a XML nodeset.
+/// @param element starting element of the nodeset.
 bool ConfigClientSetup::createOptionFromNode(
+    const std::shared_ptr<Config>& config,
     const pugi::xml_node& element,
     const std::shared_ptr<ClientConfigList>& result) const
 {
@@ -56,9 +57,13 @@ bool ConfigClientSetup::createOptionFromNode(
     std::map<std::string, std::shared_ptr<ClientGroupConfig>> groupCache;
     for (auto&& it : gcs->getXmlTree(element)) {
         const pugi::xml_node& child = it.node();
-        auto name = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_GROUP_NAME)->getXmlContent(child);
-        auto group = std::make_shared<ClientGroupConfig>(name);
-        auto forbiddenDirectories = definition->findConfigSetup<ConfigArraySetup>(ConfigVal::A_CLIENTS_GROUP_HIDDEN_LIST)->getXmlContent(child);
+        if (config) {
+            config->registerNode(child.path());
+        }
+        auto name = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_GROUP_NAME)->getXmlContent(child, config);
+        auto isAllowed = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_GROUP_ALLOWED)->getXmlContent(child, config);
+        auto group = std::make_shared<ClientGroupConfig>(name, isAllowed);
+        auto forbiddenDirectories = definition->findConfigSetup<ConfigArraySetup>(ConfigVal::A_CLIENTS_GROUP_HIDDEN_LIST)->getXmlContent(child, config);
         group->setForbiddenDirectories(forbiddenDirectories);
         EDIT_CAST(EditHelperClientGroupConfig, result)->add(group);
         groupCache[name] = group;
@@ -67,21 +72,28 @@ bool ConfigClientSetup::createOptionFromNode(
     auto&& ccs = definition->findConfigSetup<ConfigSetup>(ConfigVal::A_CLIENTS_CLIENT);
     for (auto&& it : ccs->getXmlTree(element)) {
         const pugi::xml_node& child = it.node();
+        if (config) {
+            config->registerNode(child.path());
+        }
 
-        auto flags = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_CLIENT_FLAGS)->getXmlContent(child);
-        auto group = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_GROUP)->getXmlContent(child);
-        auto ip = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_IP)->getXmlContent(child);
-        auto userAgent = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_USERAGENT)->getXmlContent(child);
-        auto captionInfoCount = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_UPNP_CAPTION_COUNT)->getXmlContent(child);
-        auto stringLimit = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_UPNP_STRING_LIMIT)->getXmlContent(child);
-        auto multiValue = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_UPNP_MULTI_VALUE)->getXmlContent(child);
-        auto fullFilter = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_UPNP_FILTER_FULL)->getXmlContent(child);
-        auto isAllowed = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_CLIENT_ALLOWED)->getXmlContent(child);
-        auto mappings = definition->findConfigSetup<ConfigDictionarySetup>(ConfigVal::A_CLIENTS_UPNP_MAP_MIMETYPE)->getXmlContent(child);
-        auto headers = definition->findConfigSetup<ConfigDictionarySetup>(ConfigVal::A_CLIENTS_UPNP_HEADERS)->getXmlContent(child);
-        auto profiles = definition->findConfigSetup<ConfigVectorSetup>(ConfigVal::A_CLIENTS_UPNP_MAP_DLNAPROFILE)->getXmlContent(child);
+        auto flags = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_CLIENT_FLAGS)->getXmlContent(child, config);
+        auto group = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_GROUP)->getXmlContent(child, config);
+        auto ip = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_IP)->getXmlContent(child, config);
+        auto userAgent = definition->findConfigSetup<ConfigStringSetup>(ConfigVal::A_CLIENTS_CLIENT_USERAGENT)->getXmlContent(child, config);
+        auto captionInfoCount = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_UPNP_CAPTION_COUNT)->getXmlContent(child, config);
+        auto stringLimit = definition->findConfigSetup<ConfigIntSetup>(ConfigVal::A_CLIENTS_UPNP_STRING_LIMIT)->getXmlContent(child, config);
+        auto multiValue = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_UPNP_MULTI_VALUE)->getXmlContent(child, config);
+        auto fullFilter = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_UPNP_FILTER_FULL)->getXmlContent(child, config);
+        auto allowCS = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_CLIENTS_CLIENT_ALLOWED);
+        auto isAllowed = allowCS->getXmlContent(child, config);
+        auto mappings = definition->findConfigSetup<ConfigDictionarySetup>(ConfigVal::A_CLIENTS_UPNP_MAP_MIMETYPE)->getXmlContent(child, config);
+        auto headers = definition->findConfigSetup<ConfigDictionarySetup>(ConfigVal::A_CLIENTS_UPNP_HEADERS)->getXmlContent(child, config);
+        auto profiles = definition->findConfigSetup<ConfigVectorSetup>(ConfigVal::A_CLIENTS_UPNP_MAP_DLNAPROFILE)->getXmlContent(child, config);
         auto matchValues = std::map<ClientMatchType, std::string>();
         for (auto&& attr : child.attributes()) {
+            if (config) {
+                config->registerNode(fmt::format("{}/attribute::{}", child.path(), attr.name()));
+            }
             auto matchType = ClientConfig::remapMatchType(attr.name());
             if (matchType != ClientMatchType::None && matchType != ClientMatchType::IP && matchType != ClientMatchType::UserAgent) {
                 matchValues[matchType] = attr.value();
@@ -90,8 +102,12 @@ bool ConfigClientSetup::createOptionFromNode(
 
         auto client = std::make_shared<ClientConfig>(flags, group, ip, userAgent, matchValues, captionInfoCount, stringLimit, multiValue, isAllowed);
         try {
-            if (groupCache.find(group) != groupCache.end())
+            if (groupCache.find(group) != groupCache.end()) {
                 client->setGroup(group, groupCache.at(group));
+                if (!allowCS->hasXmlElement(child)) {
+                    client->setAllowed(groupCache.at(group)->getAllowed());
+                }
+            }
             client->setFullFilter(fullFilter);
             if (!mappings.empty())
                 client->setMimeMappings(mappings);
@@ -116,7 +132,7 @@ void ConfigClientSetup::makeOption(
     if (arguments && arguments->find("isEnabled") != arguments->end()) {
         isEnabled = arguments->at("isEnabled") == "true";
     }
-    newOption(getXmlElement(root));
+    newOption(config, getXmlElement(root));
     setOption(config);
 }
 
@@ -213,10 +229,10 @@ bool ConfigClientSetup::updateItem(
                 return true;
             },
         },
-        // upnp multi value
+        // Allowed
         {
             { ConfigVal::A_CLIENTS_CLIENT, ConfigVal::A_CLIENTS_CLIENT_ALLOWED },
-            "upnp multi value",
+            "Allowed",
             [&](const std::shared_ptr<ClientConfig>& entry) { return fmt::to_string(entry->getAllowed()); },
             [&](const std::shared_ptr<ClientConfig>& entry, const std::shared_ptr<ConfigDefinition>& definition, ConfigVal cfg, std::string& optValue) {
                 entry->setAllowed(definition->findConfigSetup<ConfigBoolSetup>(cfg)->checkValue(optValue));
@@ -411,6 +427,32 @@ bool ConfigClientSetup::updateItem(
             return true;
         }
     }
+    static auto resultProperties = std::vector<ConfigResultProperty<ClientGroupConfig>> {
+        // Allowed
+        {
+            { ConfigVal::A_CLIENTS_GROUP, ConfigVal::A_CLIENTS_GROUP_ALLOWED },
+            "Allowed",
+            [&](const std::shared_ptr<ClientGroupConfig>& entry) { return fmt::to_string(entry->getAllowed()); },
+            [&](const std::shared_ptr<ClientGroupConfig>& entry, const std::shared_ptr<ConfigDefinition>& definition, ConfigVal cfg, std::string& optValue) {
+                entry->setAllowed(definition->findConfigSetup<ConfigBoolSetup>(cfg)->checkValue(optValue));
+                return true;
+            },
+        },
+    };
+
+    auto i = indexList.at(0);
+    for (auto&& [cfg, label, getProperty, setProperty] : resultProperties) {
+        auto index = getItemPath(indexList, cfg);
+        if (optItem == index) {
+            if (entry->getOrig())
+                config->setOrigValue(index, getProperty(entry));
+            if (setProperty(entry, definition, cfg.at(0), optValue)) {
+                auto nEntry = EDIT_CAST(EditHelperClientGroupConfig, config->getClientConfigListOption(option))->get(i);
+                log_debug("New value for Group {} {} = {}", label.data(), index, getProperty(nEntry));
+                return true;
+            }
+        }
+    }
     {
         // set forbidden directory
         auto keyIndex = getItemPath(indexList, { ConfigVal::A_CLIENTS_GROUP, ConfigVal::A_CLIENTS_GROUP_HIDE, ConfigVal::A_CLIENTS_GROUP_LOCATION });
@@ -424,7 +466,7 @@ bool ConfigClientSetup::updateItem(
             } else {
                 config->setOrigValue(keyIndex, profiles);
                 entry->setForbiddenDirectory(j, optValue);
-                log_debug("location dlna value {} {}", keyIndex, optValue);
+                log_debug("location forbidden value {} {}", keyIndex, optValue);
             }
             return true;
         }
@@ -459,11 +501,13 @@ bool ConfigClientSetup::updateDetail(
     return false;
 }
 
-std::shared_ptr<ConfigOption> ConfigClientSetup::newOption(const pugi::xml_node& optValue)
+std::shared_ptr<ConfigOption> ConfigClientSetup::newOption(
+    const std::shared_ptr<Config>& config,
+    const pugi::xml_node& optValue)
 {
     auto result = std::make_shared<ClientConfigList>();
 
-    if (!createOptionFromNode(isEnabled ? optValue : pugi::xml_node(nullptr), result)) {
+    if (!createOptionFromNode(config, isEnabled ? optValue : pugi::xml_node(nullptr), result)) {
         throw_std_runtime_error("Init {} client config failed '{}'", xpath, optValue.name());
     }
     optionValue = std::make_shared<ClientConfigListOption>(result);
