@@ -33,7 +33,7 @@
     Copyright (C) 2007 Ingo Preiml <ipreiml@edu.uni-klu.ac.at>
 */
 
-/// \file ffmpeg_handler.cc
+/// @file metadata/ffmpeg_handler.cc
 
 // Information about the stream are to be found in the structure
 // AVFormatContext, defined in libavformat/avformat.h:335
@@ -79,7 +79,7 @@ extern "C" {
 
 #define STREAM_NUMBER_OPTION "streamNumber"
 
-/// \brief Wrapper class to log FFMpeg messages
+/// @brief Wrapper class to log FFMpeg messages
 class FfmpegLogger {
 public:
     FfmpegLogger()
@@ -151,6 +151,7 @@ FfmpegHandler::FfmpegHandler(const std::shared_ptr<Context>& context)
           ConfigVal::IMPORT_LIBOPTS_FFMPEG_COMMENT_ENABLED,
           ConfigVal::IMPORT_LIBOPTS_FFMPEG_COMMENT_LIST)
     , artWorkEnabled(config->getBoolOption(ConfigVal::IMPORT_LIBOPTS_FFMPEG_ARTWORK_ENABLED))
+    , subtitleSeekSize(config->getUIntOption(ConfigVal::IMPORT_LIBOPTS_FFMPEG_SUBTITLE_SEEK_SIZE))
 {
 #if (LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58, 9, 100))
     // Register all formats and codecs
@@ -167,16 +168,36 @@ bool FfmpegHandler::isSupported(
     return mediaType == ObjectType::Audio || mediaType == ObjectType::Video;
 }
 
-/// \brief Wrapper class to interface with FFMpeg
+static ObjectType getStreamType(const AVStream* st)
+{
+    switch (as_codecpar(st)->codec_type) {
+    case AVMEDIA_TYPE_VIDEO:
+        return ObjectType::Video;
+    case AVMEDIA_TYPE_AUDIO:
+        return ObjectType::Audio;
+    case AVMEDIA_TYPE_SUBTITLE:
+    case AVMEDIA_TYPE_DATA:
+    case AVMEDIA_TYPE_ATTACHMENT:
+    case AVMEDIA_TYPE_NB:
+        return ObjectType::Unknown;
+    case AVMEDIA_TYPE_UNKNOWN:
+    default:
+        return ObjectType::Unknown;
+    }
+}
+
+/// @brief Wrapper class to interface with FFMpeg
 class FfmpegObject {
 public:
     std::string location;
     std::shared_ptr<StringConverter> sc;
+    ObjectType objType;
     AVFormatContext* pFormatCtx = nullptr;
 
     FfmpegObject(const std::shared_ptr<ConverterManager>& converterManager, const std::shared_ptr<CdsItem>& item)
         : location(item->getLocation())
         , sc(converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_FFMPEG_CHARSET, location))
+        , objType(item->getMediaType())
     {
         // ffmpeg library context
         pFormatCtx = avformat_alloc_context();
@@ -203,7 +224,10 @@ public:
             avformat_close_input(&pFormatCtx);
     }
 
-    /// \brief check if FFMpeg information is available
+    FfmpegObject(const FfmpegObject&) = delete;
+    FfmpegObject& operator=(const FfmpegObject&) = delete;
+
+    /// @brief check if FFMpeg information is available
     operator bool() const
     {
         if (!pFormatCtx)
@@ -218,7 +242,7 @@ public:
         return true;
     }
 
-    /// \brief get key value from image
+    /// @brief get key value from image
     std::string getKey(const std::string& desiredTag) const
     {
         log_debug("key: {} ", desiredTag);
@@ -324,12 +348,14 @@ bool FfmpegHandler::addFfmpegMetadataFields(
     AVDictionaryEntry* avEntry = nullptr;
     if (ffmpegObject.pFormatCtx->metadata)
         while ((avEntry = av_dict_get(ffmpegObject.pFormatCtx->metadata, "", avEntry, AV_DICT_IGNORE_SUFFIX))) {
-            result = getFfmpegMetadataField(item, ffmpegObject, avEntry, emptyProperties, emptySpecProperties) || result;
+            result = getFfmpegMetadataField(item, ffmpegObject, avEntry, item->getMediaType(), emptyProperties, emptySpecProperties) || result;
         }
     for (std::size_t stream_number = 0; stream_number < ffmpegObject.pFormatCtx->nb_streams; stream_number++) {
-        if (ffmpegObject.pFormatCtx->streams[stream_number]->metadata) {
-            while ((avEntry = av_dict_get(ffmpegObject.pFormatCtx->streams[stream_number]->metadata, "", avEntry, AV_DICT_IGNORE_SUFFIX))) {
-                result = getFfmpegMetadataField(item, ffmpegObject, avEntry, emptyProperties, emptySpecProperties) || result;
+        auto st = ffmpegObject.pFormatCtx->streams[stream_number];
+        if (st->metadata) {
+            auto streamType = getStreamType(st);
+            while ((avEntry = av_dict_get(st->metadata, "", avEntry, AV_DICT_IGNORE_SUFFIX))) {
+                result = getFfmpegMetadataField(item, ffmpegObject, avEntry, streamType, emptyProperties, emptySpecProperties) || result;
             }
         }
     }
@@ -340,6 +366,7 @@ bool FfmpegHandler::getFfmpegMetadataField(
     const std::shared_ptr<CdsItem>& item,
     const FfmpegObject& ffmpegObject,
     AVDictionaryEntry* avEntry,
+    ObjectType streamType,
     std::map<MetadataFields, bool>& emptyProperties,
     std::map<std::string, bool>& emptySpecProperties) const
 {
@@ -370,6 +397,8 @@ bool FfmpegHandler::getFfmpegMetadataField(
     if (pIt != propertyMap.end()) {
         log_debug("Identified default metadata '{}': {}", pIt->second, value);
         const auto field = pIt->first;
+        if (field == MetadataFields::M_TITLE && streamType != item->getMediaType())
+            return result;
         if (emptyProperties[field]) {
             if (field == MetadataFields::M_DATE || field == MetadataFields::M_CREATION_DATE) {
                 std::tm tmWork {};
@@ -407,7 +436,7 @@ bool FfmpegHandler::getFfmpegMetadataField(
     return result;
 }
 
-/// \brief Convert Rotation Information to 360 degree value
+/// @brief Convert Rotation Information to 360 degree value
 static double get_rotation(const std::int32_t* displaymatrix)
 {
     double rot = 0;
@@ -422,7 +451,7 @@ static double get_rotation(const std::int32_t* displaymatrix)
     return rot;
 }
 
-/// \brief Extract Artwork from media file
+/// @brief Extract Artwork from media file
 static std::vector<std::uint8_t> extractArtImage(
     const FfmpegObject& ffmpegObject,
     std::size_t imageStreamIndex)
@@ -461,15 +490,20 @@ static std::vector<std::uint8_t> extractArtImage(
         return {};
     }
 
-    log_debug("end");
+    log_debug("end APIC {}", strlen(avEntry->value));
     return std::vector<std::uint8_t>(avEntry->value, avEntry->value + strlen(avEntry->value));
 }
 
-/// \brief Extract Subtitle from media file
+/// @brief Extract Subtitle from media file
 static std::vector<std::uint8_t> extractSubtitle(
     const FfmpegObject& ffmpegObject,
-    long long subtitleStreamIndex)
+    long long subtitleStreamIndex,
+    std::size_t maxBytes = 0)
 {
+#if (LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(60, 0, 0))
+    log_debug("extractSubtitle disabled - ffmpeg seems broken");
+    return {}; // crashes when reading some subtitles
+#else
     log_debug("start");
     if (!ffmpegObject) {
         return {};
@@ -485,7 +519,8 @@ static std::vector<std::uint8_t> extractSubtitle(
 
     std::vector<std::uint8_t> result;
     // Store subtitle packets
-    while (av_read_frame(ffmpegObject.pFormatCtx, packet) >= 0) {
+    while (av_read_frame(ffmpegObject.pFormatCtx, packet) >= 0 && (maxBytes == 0 || result.size() < maxBytes)) {
+        log_vdebug("checking {} vs {}", packet->stream_index, subtitleStreamIndex);
         if (packet->stream_index == subtitleStreamIndex) {
             auto chunk = std::vector<uint8_t>(packet->data, packet->data + packet->size);
             result.reserve(result.size() + chunk.size());
@@ -493,10 +528,13 @@ static std::vector<std::uint8_t> extractSubtitle(
         }
         av_packet_unref(packet);
     }
+    av_packet_free(&packet);
+    log_debug("end {}", result.size());
     return result;
+#endif
 }
 
-/// \brief extract orientation from stream
+/// @brief extract orientation from stream
 static int getOrientation(AVStream* st)
 {
     AVDictionaryEntry* entry = nullptr;
@@ -681,7 +719,7 @@ bool FfmpegHandler::addFfmpegResourceFields(
             stResource->addAttribute(ResourceAttribute::TYPE, avcodec_get_name(codecId));
             stResource->addOption(STREAM_NUMBER_OPTION, fmt::to_string(stream_number));
 
-            auto subtitle = extractSubtitle(ffmpegObject, stream_number);
+            auto subtitle = extractSubtitle(ffmpegObject, stream_number, subtitleSeekSize);
             if (!subtitle.empty()) {
                 auto subMimetype = getContentTypeFromByteVector(subtitle);
                 log_debug("subtitle {} {}", subtitle.size(), subMimetype);
@@ -791,7 +829,7 @@ bool FfmpegHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
     // Fabricate comment
     if (item->getMetaData(MetadataFields::M_DESCRIPTION).empty())
         result = addFfmpegComment(item, ffmpegObject) || result;
-    return true;
+    return result;
 }
 
 std::unique_ptr<IOHandler> FfmpegHandler::serveContent(
