@@ -29,7 +29,7 @@
     $Id$
 */
 
-/// \file server.cc
+/// @file server.cc
 #define GRB_LOG_FAC GrbLogFacility::server
 
 #include "server.h" // API
@@ -56,6 +56,7 @@
 #include "upnp/compat.h"
 #include "upnp/conn_mgr_service.h"
 #include "upnp/cont_dir_service.h"
+#include "upnp/headers.h"
 #include "upnp/mr_reg_service.h"
 #include "upnp/upnp_common.h"
 #include "upnp/xml_builder.h"
@@ -73,6 +74,7 @@
 
 #ifdef HAVE_INOTIFY
 #include "content/inotify/autoscan_inotify.h"
+#include "content/inotify/scripting_inotify.h"
 #endif
 
 #ifdef ONLINE_SERVICES
@@ -97,7 +99,11 @@ Server::Server(std::shared_ptr<Config> config)
 {
 }
 
-void Server::init(const std::shared_ptr<ConfigDefinition>& definition, bool offln)
+void Server::init(
+    const std::shared_ptr<ConfigDefinition>& definition,
+    bool offln,
+    bool dropDatabase,
+    bool initLastFM)
 {
     offline = offln;
 
@@ -113,6 +119,15 @@ void Server::init(const std::shared_ptr<ConfigDefinition>& definition, bool offl
     mime = std::make_shared<Mime>(config);
     converterManager = std::make_shared<ConverterManager>(config);
     database = Database::createInstance(config, mime, converterManager, timer);
+
+    // special start mode
+    if (dropDatabase) {
+        database->dropTables();
+        return;
+    }
+
+    database->init();
+
     config->updateConfigFromDatabase(database);
 
     serverUDN = config->getOption(ConfigVal::SERVER_UDN);
@@ -121,11 +136,22 @@ void Server::init(const std::shared_ptr<ConfigDefinition>& definition, bool offl
     }
     aliveAdvertisementInterval = config->getIntOption(ConfigVal::SERVER_ALIVE_INTERVAL);
 
-    clientManager = std::make_shared<ClientManager>(config, database);
+    clientManager = std::make_shared<ClientManager>(config, database, self);
     sessionManager = std::make_shared<Web::SessionManager>(config, timer);
     context = std::make_shared<Context>(definition, config, clientManager, mime, database, sessionManager, converterManager);
 
     content = std::make_shared<ContentManager>(context, self, timer);
+
+#ifdef HAVE_LASTFM
+#ifndef HAVE_LASTFMLIB
+    // special start mode
+    if (initLastFM) {
+        content->initLastFM();
+        return;
+    }
+#endif
+#endif
+
     metadataService = std::make_shared<MetadataService>(context, content);
 }
 
@@ -288,6 +314,8 @@ void Server::run()
     }
 
     UpnpSetHostValidateCallback(HostValidateCallback, this);
+
+    running = true;
 }
 
 std::string Server::getIp() const
@@ -603,7 +631,7 @@ void Server::routeSubscriptionRequest(const SubscriptionRequest& request) const
     throw UpnpException(UPNP_E_BAD_REQUEST, "Service does not exist or subscriptions not supported");
 }
 
-/// \brief sendSubscriptionUpdate
+/// @brief sendSubscriptionUpdate
 void Server::sendSubscriptionUpdate(const std::string& updateString, const std::string& serviceId)
 {
     for (auto&& svc : serviceList) {
@@ -615,7 +643,9 @@ void Server::sendSubscriptionUpdate(const std::string& updateString, const std::
     }
 }
 
-std::unique_ptr<RequestHandler> Server::createRequestHandler(const char* filename, const std::shared_ptr<Quirks>& quirks) const
+std::unique_ptr<RequestHandler> Server::createRequestHandler(
+    const char* filename,
+    const std::shared_ptr<Quirks>& quirks) const
 {
     std::string link = URLUtils::urlUnescape(filename);
     log_debug("Filename: {}", filename);
@@ -654,7 +684,7 @@ std::unique_ptr<RequestHandler> Server::createRequestHandler(const char* filenam
         return std::make_unique<UIHandler>(content, upnpXmlBuilder, quirks, self);
     }
 
-#if defined(HAVE_CURL)
+#ifdef HAVE_CURL
     if (startswith(link, fmt::format("/{}", CONTENT_ONLINE_HANDLER))) {
         return std::make_unique<URLRequestHandler>(content, upnpXmlBuilder, quirks);
     }
@@ -666,9 +696,10 @@ std::unique_ptr<RequestHandler> Server::createRequestHandler(const char* filenam
 std::shared_ptr<Quirks> Server::getQuirks(const UpnpFileInfo* info, bool isWeb) const
 {
     auto ctrlPtIPAddr = std::make_shared<GrbNet>(UpnpFileInfo_get_CtrlPtIPAddr(info));
+    auto headers = std::make_shared<Headers>(info);
     // HINT: most clients do not report exactly the same User-Agent for UPnP services and file request.
     std::string userAgent = UpnpFileInfo_get_Os_cstr(info);
-    return std::make_shared<Quirks>(isWeb ? webXmlBuilder : upnpXmlBuilder, context->getClients(), ctrlPtIPAddr, std::move(userAgent));
+    return std::make_shared<Quirks>(isWeb ? webXmlBuilder : upnpXmlBuilder, context->getClients(), ctrlPtIPAddr, std::move(userAgent), headers);
 }
 
 int Server::HostValidateCallback(const char* host, void* cookie)
@@ -690,7 +721,11 @@ int Server::HostValidateCallback(const char* host, void* cookie)
     return UPNP_E_BAD_HTTPMSG;
 }
 
-int Server::GetInfoCallback(const char* filename, UpnpFileInfo* info, const void* cookie, const void** requestCookie)
+int Server::GetInfoCallback(
+    const char* filename,
+    UpnpFileInfo* info,
+    const void* cookie,
+    const void** requestCookie)
 {
     try {
         log_debug("getInfo({})", filename);
@@ -718,7 +753,11 @@ int Server::GetInfoCallback(const char* filename, UpnpFileInfo* info, const void
     }
 }
 
-UpnpWebFileHandle Server::OpenCallback(const char* filename, enum UpnpOpenFileMode mode, const void* cookie, const void* requestCookie)
+UpnpWebFileHandle Server::OpenCallback(
+    const char* filename,
+    enum UpnpOpenFileMode mode,
+    const void* cookie,
+    const void* requestCookie)
 {
     try {
         log_debug("open({})", filename);
@@ -752,7 +791,12 @@ UpnpWebFileHandle Server::OpenCallback(const char* filename, enum UpnpOpenFileMo
     }
 }
 
-int Server::ReadCallback(UpnpWebFileHandle f, char* buf, std::size_t length, const void* cookie, const void* requestCookie)
+int Server::ReadCallback(
+    UpnpWebFileHandle f,
+    char* buf,
+    std::size_t length,
+    const void* cookie,
+    const void* requestCookie)
 {
     log_debug("{} read({})", f, length);
     if (static_cast<const Server*>(cookie)->getShutdownStatus())
@@ -769,13 +813,23 @@ int Server::ReadCallback(UpnpWebFileHandle f, char* buf, std::size_t length, con
     return ioHandler ? ioHandler->read(reinterpret_cast<std::byte*>(buf), length) : 0;
 }
 
-int Server::WriteCallback(UpnpWebFileHandle f, char* buf, std::size_t length, const void* cookie, const void* requestCookie)
+int Server::WriteCallback(
+    UpnpWebFileHandle f,
+    char* buf,
+    std::size_t length,
+    const void* cookie,
+    const void* requestCookie)
 {
     log_debug("{} write not implemented({})", f, length);
     return 0;
 }
 
-int Server::SeekCallback(UpnpWebFileHandle f, off_t offset, int whence, const void* cookie, const void* requestCookie)
+int Server::SeekCallback(
+    UpnpWebFileHandle f,
+    off_t offset,
+    int whence,
+    const void* cookie,
+    const void* requestCookie)
 {
     log_debug("{} seek({}, {})", f, offset, whence);
     try {
@@ -796,7 +850,10 @@ int Server::SeekCallback(UpnpWebFileHandle f, off_t offset, int whence, const vo
     }
 }
 
-int Server::CloseCallback(UpnpWebFileHandle f, const void* cookie, const void* requestCookie)
+int Server::CloseCallback(
+    UpnpWebFileHandle f,
+    const void* cookie,
+    const void* requestCookie)
 {
     log_debug("{} close()", f);
     int retClose = 0;
